@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2011 - 2016, Phoronix Media
-	Copyright (C) 2011 - 2016, Michael Larabel
+	Copyright (C) 2011 - 2019, Phoronix Media
+	Copyright (C) 2011 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -29,18 +29,23 @@ class interactive implements pts_option_interface
 	{
 		pts_openbenchmarking::refresh_repository_lists();
 		pts_client::$display->generic_heading('Interactive Benchmarking');
-		echo 'System Hardware:' . PHP_EOL . phodevi::system_hardware(true) . (phodevi::read_property('motherboard', 'serial-number') != null ? PHP_EOL . 'System Serial Number: ' . phodevi::read_property('motherboard', 'serial-number') : null) . PHP_EOL . PHP_EOL . PHP_EOL;
+		echo phodevi::system_centralized_view();
+		echo PHP_EOL . (phodevi::read_property('motherboard', 'serial-number') != null ? PHP_EOL . 'System Serial Number: ' . phodevi::read_property('motherboard', 'serial-number') : null) . PHP_EOL . PHP_EOL;
 		$reboot_on_exit = false;
 
 		do
 		{
+			$only_show_available_cached_tests = pts_network::internet_support_available() == false;
 			$options = array(
-				'RUN_TEST' => 'Run A Test',
-				'RUN_SUITE' => 'Run A Suite [A Collection Of Tests]',
-				'RUN_SYSTEM_TEST' => 'Run Complex System Test',
+				'RUN_TEST' => 'Run A Test / Benchmark',
+				'RUN_SUITE' => 'Run A Suite      [A Collection Of Tests]',
+				'RUN_STRESS_TEST' => 'Run A Stress Test      [Concurrent Benchmarks For Heavy System Load]',
 				'SHOW_INFO' => 'Show System Hardware / Software Information',
-				'SHOW_SENSORS' => 'Show Auto-Detected System Sensors',
-				'SET_RUN_COUNT' => 'Set Test Run Repetition'
+				'SHOW_SENSORS' => 'Show Available System Sensors',
+				'LIST_TESTS' => 'List Available Tests',
+				'LIST_RECOMMENDED_TESTS' => 'List Recommended Tests',
+			//	'SET_RUN_COUNT' => 'Set Test Run Repetition',
+				'SEARCH' => 'Search Tests / Suites / Results'
 				);
 
 			if(count(pts_client::saved_test_results()) > 0)
@@ -54,13 +59,18 @@ class interactive implements pts_option_interface
 			switch($response)
 			{
 				case 'RUN_TEST':
-					$supported_tests = pts_openbenchmarking::available_tests();
+					$supported_tests = pts_openbenchmarking::available_tests(!$only_show_available_cached_tests, false, false, false, $only_show_available_cached_tests);
 					$supported_tests = pts_types::identifiers_to_test_profile_objects($supported_tests, false, true);
 					$longest_title_length = 0;
 
 					foreach($supported_tests as $i => &$test_profile)
 					{
-						if($test_profile->get_title() == null)
+						if($test_profile->get_title() == null || $test_profile->get_license() == 'Retail')
+						{
+							unset($supported_tests[$i]);
+							continue;
+						}
+						if(!pts_test_run_manager::test_profile_system_compatibility_check($test_profile))
 						{
 							unset($supported_tests[$i]);
 							continue;
@@ -80,16 +90,59 @@ class interactive implements pts_option_interface
 					$supported_tests = $t;
 					asort($supported_tests);
 
-					$tests_to_run = pts_user_io::prompt_text_menu('Select Test', $supported_tests, true, true);
+					$tests_to_run = pts_user_io::prompt_text_menu('Select Test(s)', $supported_tests, true, true);
 					$tests_to_run = explode(',', $tests_to_run);
 					pts_test_installer::standard_install($tests_to_run);
 					$run_manager = new pts_test_run_manager(false, 2);
 					$run_manager->standard_run($tests_to_run);
 					if($run_manager != false)
 					{
-						pts_client::display_web_page(PTS_SAVE_RESULTS_PATH . $run_manager->get_file_name() . '/index.html', null, true, true);
+						pts_client::display_result_view($run_manager->result_file, false);
 					}
 					break;
+				case 'RUN_STRESS_TEST':
+					$supported_tests = pts_openbenchmarking::available_tests(!$only_show_available_cached_tests, false, false, false, $only_show_available_cached_tests);
+					$supported_tests = pts_types::identifiers_to_test_profile_objects($supported_tests, false, true);
+					$longest_title_length = 0;
+
+					foreach($supported_tests as $i => &$test_profile)
+					{
+						if($test_profile->get_title() == null || $test_profile->get_license() == 'Retail')
+						{
+							unset($supported_tests[$i]);
+							continue;
+						}
+						if(!pts_test_run_manager::test_profile_system_compatibility_check($test_profile))
+						{
+							unset($supported_tests[$i]);
+							continue;
+						}
+
+						$longest_title_length = max($longest_title_length, strlen($test_profile->get_title()));
+					}
+
+					$t = array();
+					foreach($supported_tests as $i => &$test_profile)
+					{
+						if($test_profile instanceof pts_test_profile)
+						{
+							$t[$test_profile->get_identifier()] = sprintf('%-' . ($longest_title_length + 1) . 'ls - %-10ls', $test_profile->get_title(), $test_profile->get_test_hardware_type());
+						}
+					}
+					$supported_tests = $t;
+					asort($supported_tests);
+
+					$tests_to_run = pts_user_io::prompt_text_menu('Select Test(s)', $supported_tests, true, true);
+					$tests_to_run = explode(',', $tests_to_run);
+
+					$concurrent_runs = pts_user_io::prompt_user_input('Number of tests to run concurrently');
+					putenv('PTS_CONCURRENT_TEST_RUNS=' . trim($concurrent_runs));
+					$minutes_loop_time = pts_user_io::prompt_user_input('Number of minutes to stress run');
+					putenv('TOTAL_LOOP_TIME=' . trim($minutes_loop_time));
+
+					pts_test_installer::standard_install($tests_to_run);
+					pts_client::execute_command('stress_run', $tests_to_run);
+					break;
 				case 'RUN_SUITE':
 					$possible_suites = pts_openbenchmarking::available_suites();
 
@@ -109,44 +162,41 @@ class interactive implements pts_option_interface
 				case 'SELECT_DRIVE_MOUNT':
 					self::select_drive_mount();
 					break;
-				case 'RUN_SYSTEM_TEST':
-					pts_client::$display->generic_heading('System Test');
-					$system_tests = array('apache', 'c-ray', 'ramspeed', 'postmark');
-					pts_test_installer::standard_install($system_tests);
-					$run_manager = new pts_test_run_manager(false, 2);
-					$run_manager->standard_run($system_tests);
-					if($run_manager != false)
-					{
-						pts_client::display_web_page(PTS_SAVE_RESULTS_PATH . $run_manager->get_file_name() . '/index.html', null, true, true);
-					}
+				case 'SEARCH':
+					pts_client::execute_command('search');
 					break;
 				case 'SHOW_INFO':
-					pts_client::$display->generic_heading('System Software / Hardware Information');
-					echo 'Hardware:' . PHP_EOL . phodevi::system_hardware(true) . PHP_EOL . PHP_EOL;
-					echo 'Software:' . PHP_EOL . phodevi::system_software(true) . PHP_EOL . PHP_EOL;
+					pts_client::execute_command('system_info');
 					break;
 				case 'SHOW_SENSORS':
 					pts_client::execute_command('system_sensors');
 					break;
+				case 'LIST_TESTS':
+					pts_client::execute_command('list_available_tests');
+					break;
+				case 'LIST_RECOMMENDED_TESTS':
+					pts_client::execute_command('list_recommended_tests');
+					break;
 				case 'SET_RUN_COUNT':
 					$run_count = pts_user_io::prompt_user_input('Set the minimum number of times each test should repeat', false);
 					putenv('FORCE_TIMES_TO_RUN=' . trim($run_count));
 					break;
 				case 'BACKUP_RESULTS_TO_USB':
 					pts_client::$display->generic_heading('Backing Up Test Results');
-
-					foreach(pts_file_io::glob('/media/*') as $media_dir)
+					$writable_backup_locations = array();
+					foreach(array_merge(pts_file_io::glob('/media/*'), pts_file_io::glob('/run/media/*/*')) as $media_dir)
 					{
-						if(!is_writable($media_dir))
+						if(is_writable($media_dir))
 						{
-							echo PHP_EOL . $media_dir . ' is not writable.' . PHP_EOL;
-							continue;
+							$writable_backup_locations[] = $media_dir;
 						}
-
-						echo PHP_EOL . 'Writing Test Results To: ' . $media_dir . PHP_EOL;
-						pts_file_io::copy(PTS_SAVE_RESULTS_PATH, $media_dir . '/');
-						break;
 					}
+
+					$backup_location = pts_user_io::prompt_text_menu('Select Backup Location', $writable_backup_locations);
+					$backup_location .= '/phoronix-test-suite-test-results/';
+					pts_file_io::mkdir($backup_location);
+					echo PHP_EOL . pts_client::cli_just_bold('Writing Test Results To: ') . $backup_location . PHP_EOL;
+					pts_file_io::copy(PTS_SAVE_RESULTS_PATH, $backup_location . '/');
 					break;
 			}
 			echo PHP_EOL . PHP_EOL;
@@ -161,7 +211,7 @@ class interactive implements pts_option_interface
 				exec('umount /media/pts-auto-mount 2>&1');
 			}
 
-			exec('reboot');
+			phodevi::reboot();
 		}
 	}
 	private static function select_drive_mount()
