@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2017, Phoronix Media
-	Copyright (C) 2008 - 2017, Michael Larabel
+	Copyright (C) 2008 - 2019, Phoronix Media
+	Copyright (C) 2008 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -52,13 +52,9 @@ class refresh_graphs implements pts_option_interface
 			$t[] = array(pts_client::cli_just_bold('Graph Size: '), $graph_bytes . ' Bytes');
 			echo pts_user_io::display_text_table($t) . PHP_EOL;
 			echo PHP_EOL . 'The ' . $identifier . ' result file graphs have been refreshed.' . PHP_EOL;
-			pts_client::display_web_page(PTS_SAVE_RESULTS_PATH . $identifier . '/index.html');
+			pts_client::display_result_view($identifier, false);
 		}
 	}
-	public static function invalid_command($passed_args = null)
-	{
-		pts_tests::invalid_command_helper($passed_args);
-	}
 }
 
 ?>
