@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2009 - 2016, Phoronix Media
-	Copyright (C) 2009 - 2016, Michael Larabel
+	Copyright (C) 2009 - 2019, Phoronix Media
+	Copyright (C) 2009 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -69,11 +69,7 @@ class reorder_result_file implements pts_option_interface
 
 		$result_file->reorder_runs($sorted_identifiers);
 		pts_client::save_test_result($result_file->get_file_location(), $result_file->get_xml());
-		pts_client::display_web_page(PTS_SAVE_RESULTS_PATH . $args[0] . '/index.html');
-	}
-	public static function invalid_command($passed_args = null)
-	{
-		pts_tests::recently_saved_results();
+		pts_client::display_result_view($result_file, false);
 	}
 }
 
