@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2017, Phoronix Media
-	Copyright (C) 2008 - 2017, Michael Larabel
+	Copyright (C) 2008 - 2019, Phoronix Media
+	Copyright (C) 2008 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -31,10 +31,6 @@ class show_result implements pts_option_interface
 		new pts_argument_check(0, array('pts_types', 'is_result_file'), null)
 		);
 	}
-	public static function invalid_command($passed_args = null)
-	{
-		pts_tests::recently_saved_results();
-	}
 	public static function run($r)
 	{
 		$URL = PTS_SAVE_RESULTS_PATH . $r[0];
@@ -44,15 +40,7 @@ class show_result implements pts_option_interface
 			pts_client::generate_result_file_graphs($r[0], PTS_SAVE_RESULTS_PATH . $r[0] . '/');
 		}
 
-		if(pts_client::read_env('DISPLAY') == false)
-		{
-			$rf = new pts_result_file($r[0]);
-			echo pts_result_file_output::result_file_to_text($rf, pts_client::terminal_width());
-		}
-		else
-		{
-			pts_client::display_web_page($URL . '/index.html', null, false, true);
-		}
+		pts_client::display_result_view($r[0], false);
 	}
 }
 
