@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2017, Phoronix Media
-	Copyright (C) 2008 - 2017, Michael Larabel
+	Copyright (C) 2008 - 2019, Phoronix Media
+	Copyright (C) 2008 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -22,6 +22,17 @@
 
 class pts_tests
 {
+	public static $extra_env_vars = null;
+	protected static $override_test_script_execution_handler = false;
+
+	public static function add_extra_env_var($name, $value)
+	{
+		self::$extra_env_vars[$name] = $value;
+	}
+	public static function clear_extra_env_vars()
+	{
+		self::$extra_env_vars = array();
+	}
 	public static function installed_tests()
 	{
 		$cleaned_tests = array();
@@ -68,11 +79,15 @@ class pts_tests
 
 		return $local_suites;
 	}
+	public static function all_suites($only_show_maintained_suites = false)
+	{
+		return array_merge(pts_openbenchmarking::available_suites(false, $only_show_maintained_suites), pts_tests::local_suites());
+	}
 	public static function scan_for_error($log_file, $strip_string)
 	{
 		$error = null;
 
-		foreach(array('fatal error', 'error:', 'error while loading', 'undefined reference', 'returned 1 exit status', 'not found', 'child process excited with status', 'error opening archive', 'failed to load') as $error_string)
+		foreach(array('fatal error', 'error:', 'error while loading', 'undefined reference', ': cannot find -l', 'returned 1 exit status', 'not found', 'child process excited with status', 'error opening archive', 'failed to load', 'fatal', 'illegal argument') as $error_string)
 		{
 			if(($e = strripos($log_file, $error_string)) !== false)
 			{
@@ -91,6 +106,7 @@ class pts_tests
 				if(isset($log_file[8]) && !isset($log_file[144]) && strpos($log_file, PHP_EOL) === false)
 				{
 					$error = $log_file;
+					break;
 				}
 			}
 		}
@@ -147,19 +163,30 @@ class pts_tests
 	public static function extra_environmental_variables(&$test_profile)
 	{
 		$extra_vars = array();
+
+		if(is_array(self::$extra_env_vars))
+		{
+			$extra_vars = self::$extra_env_vars;
+		}
+
 		$extra_vars['HOME'] = $test_profile->get_install_dir();
+		$extra_vars['DEBUG_HOME'] = $test_profile->get_install_dir();
 		$extra_vars['PATH'] = "\$PATH";
 		$extra_vars['LC_ALL'] = '';
 		$extra_vars['LC_NUMERIC'] = '';
 		$extra_vars['LC_CTYPE'] = '';
 		$extra_vars['LC_MESSAGES'] = '';
-		$extra_vars['LANG'] = '';
+		$extra_vars['LANG'] = 'en_US.utf8';
 		$extra_vars['PHP_BIN'] = PHP_BIN;
 
 		// Safe-guards to try to ensure more accurate testing
 		$extra_vars['vblank_mode'] = '0'; // Avoid sync to vblank with the open-source drivers
+		$extra_vars['MESA_VK_WSI_PRESENT_MODE'] = 'immediate'; // https://cgit.freedesktop.org/mesa/mesa/commit/?id=a182adfd83ad00e326153b00a725a014e0359bf0
 		$extra_vars['__GL_SYNC_TO_VBLANK'] = '0'; // Avoid sync to vblank with the NVIDIA binary drivers
 		$extra_vars['CCACHE_DISABLE'] = '1'; // Should avoid ccache being used in compiler tests
+		$extra_vars['OMPI_ALLOW_RUN_AS_ROOT'] = '1'; // Tests with mpirun should use --allow-run-as-root but otherwise this fallback
+		$extra_vars['OMPI_ALLOW_RUN_AS_ROOT_CONFIRM'] = '1'; // Tests with mpirun should use --allow-run-as-root but otherwise this fallback
+
 
 		foreach($test_profile->extended_test_profiles() as $i => $this_test_profile)
 		{
@@ -170,7 +197,7 @@ class pts_tests
 
 			if(is_dir($this_test_profile->get_install_dir()))
 			{
-				$extra_vars['PATH'] = $this_test_profile->get_install_dir() . ':' . $extra_vars['PATH'];
+				$extra_vars['PATH'] = $this_test_profile->get_install_dir() . pts_client::get_path_separator() . $extra_vars['PATH'];
 				$extra_vars['TEST_' . strtoupper(str_replace('-', '_', $this_test_profile->get_identifier_base_name()))] = $this_test_profile->get_install_dir();
 			}
 		}
@@ -184,7 +211,7 @@ class pts_tests
 		if(isset($extra_vars_append['PATH']))
 		{
 			// Special case variable where you likely want the two merged rather than overwriting
-			$extra_vars['PATH'] = $extra_vars_append['PATH'] . (substr($extra_vars_append['PATH'], -1) != ':' ? ':' : null) . $extra_vars['PATH'];
+			$extra_vars['PATH'] = $extra_vars_append['PATH'] . (substr($extra_vars_append['PATH'], -1) != pts_client::get_path_separator() ? pts_client::get_path_separator() : null) . $extra_vars['PATH'];
 			unset($extra_vars_append['PATH']);
 		}
 
@@ -196,7 +223,7 @@ class pts_tests
 		$result = null;
 		$test_directory = $test_profile->get_install_dir();
 		pts_file_io::mkdir($test_directory, 0777, true);
-		$os_postfix = '_' . strtolower(phodevi::operating_system());
+		$os_postfix = '_' . strtolower(phodevi::os_under_test());
 		$test_profiles = array($test_profile);
 
 		if($use_ctp)
@@ -204,7 +231,14 @@ class pts_tests
 			$test_profiles = array_merge($test_profiles, $test_profile->extended_test_profiles());
 		}
 
-		if(pts_client::executable_in_path('bash'))
+		$use_phoroscript = phodevi::is_windows();
+		if(phodevi::is_windows() && is_executable('C:\cygwin64\bin\bash.exe'))
+		{
+			$sh = 'C:\cygwin64\bin\bash.exe';
+			$use_phoroscript = false;
+			$extra_vars['PATH'] = $extra_vars['PATH'] . ';C:\cygwin64\bin';
+		}
+		else if(pts_client::executable_in_path('bash'))
 		{
 			$sh = 'bash';
 		}
@@ -224,15 +258,41 @@ class pts_tests
 					pts_client::$display->test_run_message($print_string);
 				}
 
-				if(phodevi::is_windows() || pts_client::read_env('USE_PHOROSCRIPT_INTERPRETER') != false)
+				if(self::$override_test_script_execution_handler && is_callable(self::$override_test_script_execution_handler))
 				{
-					$phoroscript = new pts_phoroscript_interpreter($run_file, $extra_vars, $test_directory);
-					$phoroscript->execute_script($pass_argument);
-					$this_result = null;
+					$this_result = call_user_func(self::$override_test_script_execution_handler, $test_directory, $sh, $run_file, $pass_argument, $extra_vars, $this_test_profile);
 				}
-				else
+
+				// if override_test_script_execution_handler returned -1, fallback to using normal script handler
+				if(!isset($this_result) || $this_result == '-1')
 				{
-					$this_result = pts_client::shell_exec('cd ' .  $test_directory . ' && ' . $sh . ' ' . $run_file . ' ' . $pass_argument . ' 2>&1', $extra_vars);
+					if($use_phoroscript || pts_client::read_env('USE_PHOROSCRIPT_INTERPRETER') != false)
+					{
+						echo PHP_EOL . 'Falling back to experimental PhoroScript code path...' . PHP_EOL;
+						$phoroscript = new pts_phoroscript_interpreter($run_file, $extra_vars, $test_directory);
+						$phoroscript->execute_script($pass_argument);
+						$this_result = null;
+					}
+					else if(phodevi::is_windows())
+					{
+						$host_env = $_SERVER;
+						unset($host_env['argv']);
+						$descriptorspec = array(0 => array('pipe', 'r'), 1 => array('pipe', 'w'), 2 => array('pipe', 'w'));
+						$test_process = proc_open($sh . ' "' . $run_file . '" ' . $pass_argument . (phodevi::is_windows() && false ? '' : ' 2>&1'), $descriptorspec, $pipes, $test_directory, array_merge($host_env, pts_client::environmental_variables(), $extra_vars));
+
+						if(is_resource($test_process))
+						{
+							//echo proc_get_status($test_process)['pid'];
+							$this_result = stream_get_contents($pipes[1]);
+							fclose($pipes[1]);
+							fclose($pipes[2]);
+							$return_value = proc_close($test_process);
+						}
+					}
+					else
+					{
+						$this_result = pts_client::shell_exec('cd ' .  $test_directory . (phodevi::is_windows() ? '; ' : ' && ') . $sh . ' ' . $run_file . ' ' . $pass_argument . (phodevi::is_windows() ? '' : ' 2>&1'), $extra_vars);
+					}
 				}
 
 				if(trim($this_result) != null)
@@ -244,6 +304,15 @@ class pts_tests
 
 		return $result;
 	}
+	public static function override_script_test_execution_handler($to_call)
+	{
+		if(is_callable($to_call))
+		{
+			self::$override_test_script_execution_handler = $to_call;
+			return true;
+		}
+		return false;
+	}
 	public static function update_test_install_xml(&$test_profile, $this_duration = 0, $is_install = false, $compiler_data = null, $install_footnote = null)
 	{
 		// Refresh/generate an install XML for pts-install.xml
@@ -254,13 +323,19 @@ class pts_tests
 		$xml_writer = new nye_XmlWriter('file://' . PTS_USER_PATH . 'xsl/' . 'pts-test-installation-viewer.xsl');
 
 		$test_duration = $test_profile->test_installation->get_average_run_time();
-		if(!is_numeric($test_duration) && !$is_install)
+
+		// When run count is higher due to forcing it with env vars, don't factor it into average to throw out the numbering...
+		$record_avg_run_time = $test_profile->get_default_times_to_run() == $test_profile->get_times_to_run();
+		if($record_avg_run_time)
 		{
-			$test_duration = $this_duration;
-		}
-		if(!$is_install && is_numeric($this_duration) && $this_duration > 0)
-		{
-			$test_duration = ceil((($test_duration * $test_profile->test_installation->get_run_count()) + $this_duration) / ($test_profile->test_installation->get_run_count() + 1));
+			if(!is_numeric($test_duration) && !$is_install)
+			{
+				$test_duration = $this_duration;
+			}
+			if(!$is_install && is_numeric($this_duration) && $this_duration > 0)
+			{
+				$test_duration = ceil((($test_duration * $test_profile->test_installation->get_run_count()) + $this_duration) / ($test_profile->test_installation->get_run_count() + 1));
+			}
 		}
 
 		$compiler_data = $is_install ? $compiler_data : $test_profile->test_installation->get_compiler_data();
@@ -304,53 +379,139 @@ class pts_tests
 
 		$xml_writer->saveXMLFile($test_profile->get_install_dir() . 'pts-install.xml');
 	}
-	public static function invalid_command_helper($passed_args)
+	public static function invalid_command_helper($passed_args, &$argument_checks)
 	{
+		$supports_passing_a_test = false;
+		foreach($argument_checks as $check)
+		{
+			if($check->get_function_check_type() == 'Test' || strpos($check->get_function_check_type(), 'Test |') !== false)
+			{
+				$supports_passing_a_test = true;
+			}
+		}
+
 		$showed_recent_results = self::recently_saved_results();
 
+		if($supports_passing_a_test)
+		{
+			$tests_to_show = array_keys(pts_openbenchmarking_client::new_and_recently_updated_tests(14, 20));
+			$tests_to_show_title = 'New + Updated Tests';
+
+			if(count($tests_to_show) < 2)
+			{
+				$tests_to_show = array_keys(pts_openbenchmarking_client::most_popular_tests(20));
+				$tests_to_show_title = 'Popular Tests';
+			}
+
+			if(count($tests_to_show) > 3)
+			{
+				$longest_test = strlen(pts_strings::find_longest_string($tests_to_show)) + 3;
+				$terminal_width = pts_client::terminal_width();
+				$tests_per_line = floor($terminal_width / $longest_test);
+				shuffle($tests_to_show);
+				$tests_to_show = array_slice($tests_to_show, 0, min(count($tests_to_show), $tests_per_line * 2 -1));
+
+				echo pts_client::cli_just_bold($tests_to_show_title . ':') . PHP_EOL;
+				$i = 0;
+				foreach($tests_to_show as $test)
+				{
+					if($i % $tests_per_line == 0)
+					{
+						echo '   ';
+					}
+					echo $test . str_repeat(' ', $longest_test - strlen($test));
+
+					$i++;
+					if($i % $tests_per_line == 0 || $i == count($tests_to_show))
+					{
+						echo PHP_EOL;
+					}
+				}
+			}
+		}
+
+		if(count($result_uploads = pts_openbenchmarking::result_uploads_from_this_ip()) > 0)
+		{
+			echo PHP_EOL . pts_client::cli_just_bold('Recent OpenBenchmarking.org Results From This IP:') . PHP_EOL;
+			$t = array();
+			foreach($result_uploads as $id => $title)
+			{
+				$t[] = array(pts_client::cli_colored_text($id, 'gray', true), $title);
+
+				if(count($t) == 5)
+				{
+					break;
+				}
+			}
+			echo pts_user_io::display_text_table($t, '   ') . PHP_EOL . PHP_EOL;
+		}
+
+		$similar_tests = array();
 		if(!empty($passed_args))
 		{
-			$arg_soundex = soundex($passed_args);
-			$similar_tests = array();
-
-			foreach(pts_openbenchmarking::linked_repositories() as $repo)
+			foreach(pts_arrays::to_array($passed_args) as $passed_arg)
 			{
-				$repo_index = pts_openbenchmarking::read_repository_index($repo);
+				$arg_soundex = soundex($passed_arg);
+				$arg_save_identifier_like = pts_test_run_manager::clean_save_name($passed_arg);
 
-				foreach(array('tests', 'suites') as $type)
+				foreach(pts_openbenchmarking::linked_repositories() as $repo)
 				{
-					if(isset($repo_index[$type]) && is_array($repo_index[$type]))
+					$repo_index = pts_openbenchmarking::read_repository_index($repo);
+
+					foreach(array('tests', 'suites') as $type)
 					{
-						foreach(array_keys($repo_index[$type]) as $identifier)
+						if(isset($repo_index[$type]) && is_array($repo_index[$type]))
 						{
-							if(soundex($identifier) == $arg_soundex)
+							foreach(array_keys($repo_index[$type]) as $identifier)
 							{
-								$similar_tests[] = array('- ' . $repo . '/' . $identifier, ' [' . ucwords(substr($type, 0, -1)) . ']');
+								if(soundex($identifier) == $arg_soundex)
+								{
+									pts_arrays::unique_push($similar_tests, array($identifier, ' [' . ucwords(substr($type, 0, -1)) . ']'));
+								}
+								else if(isset($passed_arg[3]) && strpos($identifier, $passed_arg) !== false)
+								{
+									pts_arrays::unique_push($similar_tests, array($identifier, ' [' . ucwords(substr($type, 0, -1)) . ']'));
+								}
 							}
 						}
 					}
 				}
-			}
 
-			foreach(pts_client::saved_test_results() as $result)
-			{
-				if(soundex($result) == $arg_soundex)
+				foreach(pts_client::saved_test_results() as $result)
 				{
-					$similar_tests[] = array('- ' . $result, ' [Test Result]');
+					if(soundex($result) == $arg_soundex || (isset($passed_arg[3]) && strpos($identifier, $arg_save_identifier_like) !== false))
+					{
+						pts_arrays::unique_push($similar_tests, array($result, ' [Test Result]'));
+					}
+				}
+
+				if(strpos($passed_arg, '-') !== false)
+				{
+					$possible_identifier = str_replace('-', '', $passed_arg);
+					if(pts_test_profile::is_test_profile($possible_identifier))
+					{
+						pts_arrays::unique_push($similar_tests, array($possible_identifier, ' [Test]'));
+					}
+				}
+				if($passed_arg != ($possible_identifier = strtolower($passed_arg)))
+				{
+					if(pts_test_profile::is_test_profile($possible_identifier))
+					{
+						pts_arrays::unique_push($similar_tests, array($possible_identifier, ' [Test]'));
+					}
 				}
 			}
-
-			if(count($similar_tests) > 0)
+		}
+		if(count($similar_tests) > 0)
+		{
+			echo pts_client::cli_just_bold('Possible Suggestions:') . PHP_EOL;
+			//$similar_tests = array_unique($similar_tests);
+			if(isset($similar_tests[12]))
 			{
-				echo 'Possible Suggestions:' . PHP_EOL;
-
-				if(isset($similar_tests[12]))
-				{
-					// lots of tests... trim it down
-					$similar_tests = array_rand($similar_tests, 12);
-				}
-				echo pts_user_io::display_text_table($similar_tests) . PHP_EOL . PHP_EOL;
+				// lots of tests... trim it down
+				$similar_tests = array_rand($similar_tests, 12);
 			}
+			echo pts_user_io::display_text_table($similar_tests, '- ') . PHP_EOL . PHP_EOL;
 		}
 
 		if($showed_recent_results == false)
@@ -358,10 +519,10 @@ class pts_tests
 			echo 'See available tests to run by visiting OpenBenchmarking.org or running:' . PHP_EOL . PHP_EOL;
 			echo '    phoronix-test-suite list-tests' . PHP_EOL . PHP_EOL;
 			echo 'Tests can be installed by running:' . PHP_EOL . PHP_EOL;
-			echo '    phoronix-test-suite install <test-name>' . PHP_EOL;
+			echo '    phoronix-test-suite install <test-name>' . PHP_EOL . PHP_EOL;
 		}
 	}
-	public static function recently_saved_results()
+	public static function recently_saved_results($extra_space = null)
 	{
 		$recent_results = pts_tests::test_results_by_date();
 
@@ -376,8 +537,8 @@ class pts_tests
 				$days = floor(($current_time - $m_time) / 86400);
 				$recent_result = sprintf('%-' . $res_length . 'ls [%-ls]', $recent_result, ($days == 0 ? 'Today' : pts_strings::days_ago_format_string($days) . ' old'));
 			}
-			echo PHP_EOL . pts_client::cli_just_bold('Recently Saved Test Results:') . PHP_EOL;
-			echo pts_user_io::display_text_list($recent_results) . PHP_EOL;
+			echo PHP_EOL . $extra_space . pts_client::cli_just_bold('Recently Saved Test Results:') . PHP_EOL;
+			echo pts_user_io::display_text_list($recent_results, $extra_space . '   ') . PHP_EOL;
 			return true;
 		}
 
