@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2009 - 2017, Phoronix Media
-	Copyright (C) 2009 - 2017, Michael Larabel
+	Copyright (C) 2009 - 2019, Phoronix Media
+	Copyright (C) 2009 - 2019, Michael Larabel
 	phodevi.php: The object for interacting with the PTS device framework
 
 	This program is free software; you can redistribute it and/or modify
@@ -125,6 +125,21 @@ class phodevi extends phodevi_base
 
 		return $selected;
 	}
+	public static function is_sensor_supported($sensor)
+	{
+		$supported = false;
+		$sensors = self::supported_sensors();
+		foreach($sensors as $s)
+		{
+			if($s[0] == $sensor[0] && $s[1] == $sensor[1])
+			{
+				$supported = true;
+				break;
+			}
+		}
+
+		return $supported;
+	}
 	public static function supported_sensors($limit_sensors = false)
 	{
 		static $supported_sensors = null;
@@ -185,6 +200,37 @@ class phodevi extends phodevi_base
 		$sensor = array($sensor_object->get_type(), $sensor_object->get_sensor(), get_class($sensor_object));
 		return self::read_sensor_unit($sensor);
 	}
+	public static function read_sensor_object_unit_short(&$sensor_object)
+	{
+		$sensor_unit = self::read_sensor_object_unit($sensor_object);
+
+		switch($sensor_unit)
+		{
+			case 'Celsius':
+				$sensor_unit = 'C';
+				break;
+			case 'Percent':
+				$sensor_unit = '%';
+				break;
+			case 'Megabytes':
+				$sensor_unit = 'MB';
+				break;
+			case 'Megahertz':
+				$sensor_unit = 'MHz';
+				break;
+			case 'Volts':
+				$sensor_unit = 'V';
+				break;
+			case 'Kilobytes/seconds':
+				$sensor_unit = 'KBps';
+				break;
+			case 'Millivolts':
+				$sensor_unit = 'mV';
+				break;
+		}
+
+		return $sensor_unit;
+	}
 	public static function read_sensor_unit($sensor)
 	{
 		return call_user_func(array(self::$sensors[$sensor[0]][$sensor[1]], 'get_unit'));
@@ -198,7 +244,7 @@ class phodevi extends phodevi_base
 	public static function sensor_object_identifier(&$sensor_object)
 	{
 		$sensor = array($sensor_object->get_type(), $sensor_object->get_sensor(), get_class($sensor_object));
-		return self::sensor_identifier($sensor) . '.' . $sensor_object->get_instance();
+		return self::sensor_identifier($sensor) . ($sensor_object->get_instance() != 0 ? '.' . $sensor_object->get_instance() : null);
 	}
 	public static function sensor_identifier($sensor)
 	{
@@ -290,6 +336,7 @@ class phodevi extends phodevi_base
 				),
 			'Graphics' => phodevi::read_name('gpu'),
 				array(
+				'Frequency' => phodevi::read_property('gpu', 'frequency'),
 				'OpenGL' => phodevi::read_property('system', 'opengl-driver'),
 				'Vulkan' => phodevi::read_property('system', 'vulkan-driver'),
 				'OpenCL' => phodevi::read_property('system', 'opencl-driver'),
@@ -299,11 +346,13 @@ class phodevi extends phodevi_base
 				),
 			'Motherboard' => phodevi::read_name('motherboard'),
 				array(
-				'Memory' => phodevi::read_name('memory'),
+				'BIOS Version' => phodevi::read_property('motherboard', 'bios-version'),
 				'Chipset' => phodevi::read_name('chipset'),
-				'Audio' => phodevi::read_name('audoo'),
+				'Audio' => phodevi::read_name('audio'),
 				'Network' => phodevi::read_name('network'),
 				),
+			'Memory' => phodevi::read_name('memory'),
+				array(),
 			'Disk' => phodevi::read_name('disk'),
 				array(
 				'File-System' => phodevi::read_property('system', 'filesystem'),
@@ -318,6 +367,7 @@ class phodevi extends phodevi_base
 				'Display Server' => phodevi::read_property('system', 'display-server'),
 				'Compiler' => phodevi::read_property('system', 'compiler'),
 				'System Layer' => phodevi::read_property('system', 'system-layer'),
+				'Security' => phodevi::read_property('system', 'security-features'),
 				)
 			);
 
@@ -325,7 +375,7 @@ class phodevi extends phodevi_base
 		{
 			$sys_string = null;
 			$tabled = array();
-			foreach($sys as $key => $in)
+			foreach($sys as $key => &$in)
 			{
 				$space_in = 2;
 				if(is_array($in))
@@ -335,18 +385,49 @@ class phodevi extends phodevi_base
 					{
 						if(!empty($value))
 						{
-							$tabled[] = array(pts_client::cli_just_bold($key) . ':' . str_repeat(' ', (16 - strlen($key))), $value);
-							//$sys_string .= '      ' . strtoupper($key) . ':' . $value . PHP_EOL;
+							if(isset($value[64]) && strpos($value, ' + ') !== false)
+							{
+								$values = explode(' + ', $value);
+								$tabled[] = array(pts_client::cli_just_bold($key) . ':' . str_repeat(' ', (16 - strlen($key))), array_shift($values));
+								foreach($values as $value)
+								{
+									$tabled[] = array(pts_client::cli_just_bold(' '), '+ ' . $value);
+								}
+							}
+							else
+							{
+								$tabled[] = array(pts_client::cli_just_bold($key) . ':' . str_repeat(' ', (16 - strlen($key))), $value);
+								//$sys_string .= '      ' . strtoupper($key) . ':' . $value . PHP_EOL;
+							}
 						}
 					}
 				}
 				else if(true || !empty($in)) // TODO this check not needed anymore?
 				{
+					if(($x = strpos($in, ' (')))
+					{
+						$in = substr($in, 0, $x);
+					}
+
 					if(!empty($tabled))
 					{
 						$sys_string .= pts_user_io::display_text_table($tabled, '    ', 0, 17) . PHP_EOL;
 					}
-					$sys_string .= PHP_EOL . '  ' . pts_client::cli_colored_text(strtoupper($key), 'gray', true) . ': ' . str_repeat(' ', (18 - strlen($key))) . pts_client::cli_colored_text($in, 'green', true) . PHP_EOL;
+
+					if(isset($in[80]) && strpos($in, ' + ') !== false)
+					{
+						$values = explode(' + ', $in);
+						$sys_string .= PHP_EOL . '  ' . pts_client::cli_colored_text(strtoupper($key), 'gray', true) . ': ' . str_repeat(' ', (18 - strlen($key))) . pts_client::cli_colored_text(array_shift($values), 'green', true);
+						foreach($values as $value)
+						{
+							$sys_string .= PHP_EOL . str_repeat(' ', 22) . pts_client::cli_colored_text('+ ' . $value, 'green', true);
+						}
+						$sys_string .= PHP_EOL;
+					}
+					else
+					{
+						$sys_string .= PHP_EOL . '  ' . pts_client::cli_colored_text(strtoupper($key), 'gray', true) . ': ' . str_repeat(' ', (18 - strlen($key))) . pts_client::cli_colored_text($in, 'green', true) . PHP_EOL;
+					}
 				}
 
 			}
@@ -362,7 +443,7 @@ class phodevi extends phodevi_base
 	public static function system_id_string()
 	{
 		$extra = null;
-		foreach(array('CC', 'CXX', 'CFLAGS', 'CPPFLAGS', 'CXXFLAGS') as $env)
+		foreach(array('CC', 'CXX', 'CFLAGS', 'CPPFLAGS', 'CXXFLAGS', 'USE_WINE') as $env)
 		{
 			$val = getenv($env);
 
@@ -375,41 +456,85 @@ class phodevi extends phodevi_base
 		$components = array(phodevi::read_property('cpu', 'model'), phodevi::read_property('system', 'operating-system'), phodevi::read_property('system', 'compiler'), $extra);
 		return base64_encode(implode('__', $components));
 	}
-	public static function read_device_notes($device)
+	public static function read_property($device, $read_property, $strip_string = true)
 	{
-		$devices = phodevi::available_hardware_devices();
+		static $properties_table = array();
+		$value = false;
 
-		if($device != null && isset($devices[$device]))
+		if(!isset($properties_table[$device]))
 		{
-			$notes_r = call_user_func(array('phodevi_' . $devices[$device], 'device_notes'));
+			$properties_table[$device] = call_user_func(array('phodevi_' . $device, 'properties'));
+		}
+
+		if(!isset($properties_table[$device][$read_property]))
+		{
+			echo 'NOTICE: ' . $read_property . ' does not exist for ' . $device . '.' . PHP_EOL;
+		}
+
+		if(!($properties_table[$device][$read_property] instanceof phodevi_device_property))
+		{
+			return $properties_table[$device][$read_property];
+		}
+
+		$cache_code = $properties_table[$device][$read_property]->cache_code();
+		if($cache_code != phodevi::no_caching && phodevi::$allow_phodevi_caching && isset(self::$device_cache[$device][$read_property]))
+		{
+			$value = self::$device_cache[$device][$read_property];
 		}
 		else
 		{
-			$notes_r = array();
+			$dev_function_r = pts_arrays::to_array($properties_table[$device][$read_property]->get_device_function());
+			$dev_function = $dev_function_r[0];
+			$function_pass = array();
+
+			for($i = 1; $i < count($dev_function_r); $i++)
+			{
+				array_push($function_pass, $dev_function_r[$i]);
+			}
+			if(method_exists('phodevi_' . $device, $dev_function))
+			{
+				$value = call_user_func_array(array('phodevi_' . $device, $dev_function), $function_pass);
+				if(!is_array($value) && $value != null)
+				{
+					if($strip_string)
+					{
+						$value = pts_strings::strip_string($value);
+					}
+					if(function_exists('preg_replace'))
+					{
+						$value = preg_replace('/[^(\x20-\x7F)]*/','', $value);
+					}
+				}
+
+				if($cache_code != phodevi::no_caching)
+				{
+					self::$device_cache[$device][$read_property] = $value;
+
+					if($cache_code == phodevi::smart_caching)
+					{
+						// TODO: For now just copy the smart cache to other var, but come up with better yet efficient way
+						self::$smart_cache[$device][$read_property] = $value;
+					}
+				}
+			}
 		}
 
-		return is_array($notes_r) ? $notes_r : array();
+		return $value;
 	}
-	public static function read_property($device, $read_property)
+	public static function read_all_properties()
 	{
-		$value = false;
-
-		if(method_exists('phodevi_' . $device, 'read_property'))
+		$all_properties = array();
+		$components = array();
+		foreach(glob(__DIR__ . '/components/phodevi_*.php') as $file)
 		{
-			$property = call_user_func(array('phodevi_' . $device, 'read_property'), $read_property);
+			$components[] = substr(basename($file, '.php'), 8);
+		}
 
-			if(!($property instanceof phodevi_device_property))
-			{
-				return false;
-			}
-
-			$cache_code = $property->cache_code();
-
-			if($cache_code != phodevi::no_caching && phodevi::$allow_phodevi_caching && isset(self::$device_cache[$device][$read_property]))
-			{
-				$value = self::$device_cache[$device][$read_property];
-			}
-			else
+		foreach($components as $device)
+		{
+			$properties = call_user_func(array('phodevi_' . $device, 'properties'));
+			$all_properties[$device] = array();
+			foreach($properties as $id => $property)
 			{
 				$dev_function_r = pts_arrays::to_array($property->get_device_function());
 				$dev_function = $dev_function_r[0];
@@ -419,11 +544,9 @@ class phodevi extends phodevi_base
 				{
 					array_push($function_pass, $dev_function_r[$i]);
 				}
-
 				if(method_exists('phodevi_' . $device, $dev_function))
 				{
 					$value = call_user_func_array(array('phodevi_' . $device, $dev_function), $function_pass);
-
 					if(!is_array($value) && $value != null)
 					{
 						$value = pts_strings::strip_string($value);
@@ -432,22 +555,12 @@ class phodevi extends phodevi_base
 							$value = preg_replace('/[^(\x20-\x7F)]*/','', $value);
 						}
 					}
-
-					if($cache_code != phodevi::no_caching)
-					{
-						self::$device_cache[$device][$read_property] = $value;
-
-						if($cache_code == phodevi::smart_caching)
-						{
-							// TODO: For now just copy the smart cache to other var, but come up with better yet efficient way
-							self::$smart_cache[$device][$read_property] = $value;
-						}
-					}
+					$all_properties[$device][$id] = $value;
 				}
 			}
 		}
 
-		return $value;
+		return $all_properties;
 	}
 	public static function set_property($device, $set_property, $pass_args = array())
 	{
@@ -503,7 +616,7 @@ class phodevi extends phodevi_base
 		}
 
 		// OpenGL / graphics detection
-		$graphics_detection = array('NVIDIA', array('ATI', 'AMD', 'fglrx'), array('Mesa', 'SGI'));
+		$graphics_detection = array('NVIDIA', array('Mesa', 'SGI'), array('AMD'));
 		$opengl_driver = phodevi::read_property('system', 'opengl-vendor') . ' ' . phodevi::read_property('system', 'opengl-driver') . ' ' . phodevi::read_property('system', 'dri-display-driver');
 		$opengl_driver = trim(str_replace('Corporation', null, $opengl_driver)); // Prevents a possible false positive for ATI being in CorporATIon
 
@@ -588,7 +701,12 @@ class phodevi extends phodevi_base
 		// Returns the system's uptime in seconds
 		$uptime = 1;
 
-		if(is_file('/proc/uptime'))
+		if(phodevi::is_windows())
+		{
+			$uptime = trim(shell_exec('powershell "((get-date) - (gcim Win32_OperatingSystem).LastBootUpTime).TotalSeconds"'));
+			$uptime = is_numeric($uptime) && $uptime > 1 ? round($uptime) : 1;
+		}
+		else if(is_file('/proc/uptime'))
 		{
 			$uptime = pts_strings::first_in_string(pts_file_io::file_get_contents('/proc/uptime'));
 		}
@@ -647,6 +765,39 @@ class phodevi extends phodevi_base
 
 		return intval($uptime);
 	}
+	public static function reboot()
+	{
+		if(pts_client::executable_in_path('reboot'))
+		{
+			shell_exec('reboot');
+			sleep(5);
+		}
+		if(phodevi::is_windows())
+		{
+			shell_exec('shutdown /r');
+			sleep(5);
+		}
+	}
+	public static function shutdown()
+	{
+		// some systems like systemctl poweroff, others just like poweroff, but not consistent one method for all systems in testing
+		if(pts_client::executable_in_path('systemctl') && rand(0, 1) == 1)
+		{
+			// Try systemd's poweroff method first
+			shell_exec('systemctl poweroff');
+			sleep(5);
+		}
+		if(pts_client::executable_in_path('poweroff'))
+		{
+			shell_exec('poweroff');
+			sleep(5);
+		}
+		if(phodevi::is_windows())
+		{
+			shell_exec('shutdown /s');
+			sleep(5);
+		}
+	}
 	public static function cpu_arch_compatible($check_against)
 	{
 		$compatible = true;
@@ -672,6 +823,21 @@ class phodevi extends phodevi_base
 	{
 		return phodevi::is_vendor_string($product) && !pts_strings::has_in_istring($product, array('VBOX', 'QEMU', 'Virtual', 'Family', '440BX', 'VMware', ' Gen', 'Core IGP'));
 	}
+	public static function os_under_test($force_override = false, $force_value = null)
+	{
+		static $os_under_test = null;
+		if($force_override && !empty($force_value))
+		{
+			$os_under_test = $force_value;
+		}
+		else if($os_under_test == null)
+		{
+			$os_under_test = self::operating_system();
+		}
+
+		// The operating system under test
+		return $os_under_test;
+	}
 	public static function operating_system()
 	{
 		return self::$operating_system;
@@ -709,12 +875,6 @@ class phodevi extends phodevi_base
 		self::detect_graphics();
 		return self::$graphics['mesa'];
 	}
-	public static function is_ati_graphics()
-	{
-		// Detection for fglrx / old AMD blob driver, newer AMDGPU-PRO / AMDGPU should go to is_mesa_graphics
-		self::detect_graphics();
-		return self::$graphics['ati'] && pts_client::executable_in_path('amdcccle');;
-	}
 	public static function is_nvidia_graphics()
 	{
 		self::detect_graphics();
@@ -722,7 +882,17 @@ class phodevi extends phodevi_base
 	}
 	public static function is_root()
 	{
-		return phodevi::read_property('system', 'username') == 'root';
+		return phodevi::read_property('system', 'username') == 'root' || is_writable('/root');
+	}
+	public static function is_display_server_active()
+	{
+		if(phodevi::read_property('system', 'system-layer') == 'Docker')
+		{
+			// hopefully temp workaround XXX TODO
+			return false;
+		}
+
+		return pts_client::read_env('DISPLAY') != false || pts_client::read_env('WAYLAND_DISPLAY') != false || phodevi::is_windows() || phodevi::is_macosx();
 	}
 }
 
