@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2017, Phoronix Media
-	Copyright (C) 2008 - 2017, Michael Larabel
+	Copyright (C) 2008 - 2019, Phoronix Media
+	Copyright (C) 2008 - 2019, Michael Larabel
 	pho_graph.php: The core graph object that is used by the different graphing objects.
 
 	This program is free software; you can redistribute it and/or modify
@@ -35,15 +35,8 @@ abstract class pts_graph_core
 	protected $i = array(); // internal data, pts_Graph* can read-write
 	public $svg_dom = null;
 
-	// TODO: Convert below variables to using $this->[XXX]
 	protected $graph_identifiers = array();
 	protected $graph_sub_titles = array();
-	protected $graph_title;
-	protected $graph_y_title;
-	protected $is_multi_way_comparison = false;
-	private $test_identifier = null;
-	protected $value_highlights = array();
-
 	protected $test_result;
 	protected $results;
 
@@ -74,6 +67,11 @@ abstract class pts_graph_core
 		$this->i['multi_way_comparison_invert_default'] = true;
 		$this->i['support_color_branding'] = true;
 		$this->i['notes'] = array();
+		$this->i['is_multi_way_comparison'] = false;
+		$this->i['graph_y_title'] = null;
+		$this->i['graph_title'] = null;
+		$this->i['header_link'] = null;
+		$this->i['highlight_values'] = false;
 
 		// Reset of setup besides config
 		if($result_object != null)
@@ -85,24 +83,30 @@ abstract class pts_graph_core
 				$test_version = 'v' . $test_version;
 			}
 
-			$this->graph_title = trim($result_object->test_profile->get_title() . ' ' . $test_version);
+			$this->i['graph_title'] = trim($result_object->test_profile->get_title() . ' ' . $test_version);
 
-			$this->graph_y_title = $result_object->test_profile->get_result_scale_formatted();
-			$this->test_identifier = $result_object->test_profile->get_identifier();
+			$this->i['graph_y_title'] = $result_object->test_profile->get_result_scale_formatted();
+			if(($test_identifier = $result_object->test_profile->get_identifier()))
+			{
+				$this->i['header_link'] = 'https://openbenchmarking.org/test/' . $result_object->test_profile->get_identifier();
+			}
 			$this->i['graph_proportion'] = $result_object->test_profile->get_result_proportion();
 			$this->addSubTitle($result_object->get_arguments_description());
 		}
 
 		$this->update_graph_dimensions(self::$c['graph']['width'], self::$c['graph']['height'], true);
-		if(isset($extra_attributes['force_tracking_line_graph']))
+		if(!isset($extra_attributes['skip_multi_way_comparison_check']))
 		{
-			// Phoromatic result tracker
-			// TODO: investigate this check as it could cause problems... bad assumption to make
-			$this->is_multi_way_comparison = true;
-		}
-		else if(isset($result_object->test_result_buffer))
-		{
-			$this->is_multi_way_comparison = pts_render::multi_way_identifier_check($result_object->test_result_buffer->get_identifiers());
+			if(isset($extra_attributes['force_tracking_line_graph']))
+			{
+				// Phoromatic result tracker
+				// TODO: investigate this check as it could cause problems... bad assumption to make
+				$this->i['is_multi_way_comparison'] = true;
+			}
+			else if(isset($result_object->test_result_buffer))
+			{
+				$this->i['is_multi_way_comparison'] = pts_render::multi_way_identifier_check($result_object->test_result_buffer->get_identifiers());
+			}
 		}
 
 		$this->i['graph_version'] = 'Phoronix Test Suite ' . PTS_VERSION;
@@ -117,30 +121,31 @@ abstract class pts_graph_core
 		}
 		if(isset($extra_attributes['highlight_graph_values']))
 		{
-			$this->value_highlights = $extra_attributes['highlight_graph_values'];
+			$this->i['highlight_values'] = $extra_attributes['highlight_graph_values'];
 		}
 		else if(PTS_IS_CLIENT && pts_client::read_env('GRAPH_HIGHLIGHT') != false)
 		{
-			//$this->value_highlights = pts_strings::comma_explode(pts_client::read_env('GRAPH_HIGHLIGHT'));
+			//$this->i['highlight_values'] = pts_strings::comma_explode(pts_client::read_env('GRAPH_HIGHLIGHT'));
 			// We support GRAPH_HIGHLIGHT as a series of
 			// ID, ID=colorId or ID=color values, e.g.
 			// GRAPH_HIGHLIGHT="will_be_different,group1a=1,group1b=1,blue=#0000ff"
+			$this->i['highlight_values'] = array();
 			foreach(pts_strings::comma_explode(pts_client::read_env('GRAPH_HIGHLIGHT')) as $id)
 			{
 				$split = explode('=', $id);
 				if(count($split) == 2)
 				{
-					$this->value_highlights[$split[0]] = null;
+					$this->i['highlight_values'][$split[0]] = null;
 					$color = $split[1];
 					if(is_numeric($color)) // TODO clean this up with a better color check
 					{
 						$color = self::$c['color']['paint'][$color];
-						$this->value_highlights[$split[0]] = $color;
+						$this->i['highlight_values'][$split[0]] = $color;
 					}
 				}
 				else
 				{
-					$this->value_highlights[$id] = null;
+					$this->i['highlight_values'][$id] = null;
 				}
 			}
 		}
@@ -163,12 +168,27 @@ abstract class pts_graph_core
 		{
 			$this->generate_results_var();
 		}
+
+		// reverve bar paint color (first color) from being used otherwise such as in charts rendered first
+		$this->get_paint_color(0);
+	}
+	public static function get_graph_config($o1 = null, $o2 = null)
+	{
+		if(isset(self::$c[$o1][$o2]))
+		{
+			return self::$c[$o1][$o2];
+		}
+		else
+		{
+			return self::$c;
+		}
 	}
 	protected function generate_results_var()
 	{
 		$this->results = array();
-		if($this->is_multi_way_comparison)
+		if($this->i['is_multi_way_comparison'])
 		{
+			$this->test_result->test_result_buffer->adjust_precision();
 			foreach($this->test_result->test_result_buffer->buffer_items as $i => &$buffer_item)
 			{
 				if($buffer_item->get_result_value() === null)
@@ -195,8 +215,7 @@ abstract class pts_graph_core
 						$date = $identifier[0];
 						break;
 					default:
-						continue;
-						break;
+						continue 2;
 				}
 				/*if(in_array($buffer_item->get_result_identifier(), $this->graph_identifiers))
 				{
@@ -219,11 +238,12 @@ abstract class pts_graph_core
 
 			if(count($this->results) == 1)
 			{
-				$this->is_multi_way_comparison = false;
+				$this->i['is_multi_way_comparison'] = false;
 			}
 		}
 		else if(isset($this->test_result->test_result_buffer))
 		{
+			$this->test_result->test_result_buffer->adjust_precision();
 			foreach($this->test_result->test_result_buffer->buffer_items as $i => &$buffer_item)
 			{
 				if($buffer_item->get_result_value() === null)
@@ -298,6 +318,9 @@ abstract class pts_graph_core
 		$config['size']['identifiers'] = 10;
 		$config['size']['sub_headers'] = 11;
 		$config['size']['axis_headers'] = 10;
+
+		$config['style']['allow_box_plots'] = true;
+		$config['style']['bar_graphs_horizontal'] = true;
 	}
 
 	//
@@ -336,28 +359,34 @@ abstract class pts_graph_core
 	// Misc Functions
 	//
 
-	protected function get_paint_color($identifier)
+	protected function get_paint_color($identifier, $check_branding = false)
 	{
 		// For now to try to improve the color handling of line graphs, first try to use a pre-defined pool of colors until falling back to the old color code once exhausted
-		if(!isset(self::$color_cache[$identifier]))
+		if(!isset(self::$color_cache[$check_branding][$identifier]))
 		{
 			if(!empty(self::$c['color']['paint']))
 			{
-				self::$color_cache[$identifier] = array_shift(self::$c['color']['paint']);
+				self::$color_cache[$check_branding][$identifier] = array_shift(self::$c['color']['paint']);
 			}
 			else
 			{
-				self::$color_cache[$identifier] = sprintf('#%06X', mt_rand(0, 0xFFFFFF));
+				self::$color_cache[$check_branding][$identifier] = sprintf('#%06X', mt_rand(0, 0xFFFFFF));
+			}
+
+			if($check_branding)
+			{
+				self::$color_cache[$check_branding][$identifier] = self::identifier_to_branded_color($identifier, self::$color_cache[$check_branding][$identifier]);
 			}
 		}
 
-		return self::$color_cache[$identifier];
+		return self::$color_cache[$check_branding][$identifier];
 	}
 	protected function maximum_graph_value()
 	{
 		$real_maximum = 0;
 
 		$data_max = $this->test_result->test_result_buffer->get_max_value();
+		$max_precision = pts_math::get_precision($data_max);
 		if(!is_numeric($data_max))
 		{
 			if(is_array($data_max))
@@ -383,13 +412,14 @@ abstract class pts_graph_core
 
 				if($maximum > 1)
 				{
-					round($maximum);
+					pts_math::set_precision($maximum, $max_precision);
 				}
 			}
 			else
 			{
 				$maximum = (floor(round($real_maximum * $this->i['graph_max_value_multiplier']) / $this->i['mark_count']) + 1) * $this->i['mark_count'];
 				$maximum = round(ceil($maximum / $this->i['mark_count']), (0 - strlen($maximum) + 2)) * $this->i['mark_count'];
+				$maximum = pts_math::set_precision($maximum, $max_precision);
 			}
 		}
 		else
@@ -425,37 +455,12 @@ abstract class pts_graph_core
 	}
 	protected function identifier_to_branded_color($identifier, $fallback_color = null)
 	{
-		if($this->i['support_color_branding'] == false || !isset($identifier[6]))
+		if($this->i['support_color_branding'] == false || !isset($identifier[5]))
 		{
 			return $fallback_color;
 		}
 
-		// See if the result identifier matches something to be color-coded better
-		$identifier = strtolower($identifier) . ' ';
-		if(strpos($identifier, 'geforce') !== false || strpos($identifier, 'nvidia') !== false || strpos($identifier, 'quadro') !== false)
-		{
-			$paint_color = '#77b900';
-		}
-		else if(strpos($identifier, 'radeon') !== false || strpos($identifier, 'amd ') !== false || strpos($identifier, 'EPYC') !== false || strpos($identifier, 'opteron ') !== false || strpos($identifier, 'fx-') !== false || strpos($identifier, 'firepro ') !== false || strpos($identifier, 'ryzen ') !== false)
-		{
-			$paint_color = '#f1052d';
-		}
-		else if(strpos($identifier, 'intel ') !== false || strpos($identifier, 'xeon ') !== false || strpos($identifier, 'core i') !== false || strpos($identifier, 'pentium') !== false || strpos($identifier, 'celeron') !== false)
-		{
-			$paint_color = '#0b5997';
-		}
-		else if(strpos($identifier, 'bsd') !== false)
-		{
-			$paint_color = '#850000';
-		}
-		else if(strpos($identifier, 'windows ') !== false)
-		{
-			$paint_color = '#0078d7';
-		}
-		else
-		{
-			$paint_color = $fallback_color;
-		}
+		$paint_color = pts_render::identifier_to_brand_color($identifier, $fallback_color);
 
 		if($paint_color != $fallback_color && strpos($identifier, ' - '))
 		{
@@ -495,6 +500,7 @@ abstract class pts_graph_core
 	{
 		$this->i['graph_max_value'] = $this->maximum_graph_value();
 		$longest_identifier = $this->test_result->test_result_buffer->get_longest_identifier();
+		$se_width = self::text_string_width('SE +/- 0.04, N = 3', $this->i['identifier_size'] - 1);
 
 		// Make room for tick markings, left hand side
 		if($this->i['iveland_view'] == false)
@@ -515,22 +521,22 @@ abstract class pts_graph_core
 		{
 			if($this->i['graph_orientation'] == 'HORIZONTAL')
 			{
-				if($this->is_multi_way_comparison && count($this->results) > 1)
+				if($this->i['is_multi_way_comparison'] && count($this->results) > 1)
 				{
 					$longest_r = $longest_identifier;
 					$longest_r = explode(' - ', $longest_r);
-					$plus_extra = 0;
+					$plus_extra = 22;
 
 					if(count($longest_r) > 1)
 					{
-						$plus_extra = count($longest_r) * $this->i['identifier_size'] * 1.2;
+						$plus_extra += floor(count($longest_r) * $this->i['identifier_size'] * 1.2);
 					}
 
 					$longest_identifier_width = self::text_string_width($this->i['graph_max_value'], $this->i['identifier_size']) + 60 + $plus_extra;
 				}
 				else
 				{
-					$longest_identifier_width = ceil(self::text_string_width($longest_identifier, $this->i['identifier_size']) * 1.09) + 12;
+					$longest_identifier_width = max($se_width, ceil(self::text_string_width($longest_identifier, $this->i['identifier_size']) * 1.09) + 16);
 				}
 
 				$longest_identifier_max = ceil(($this->i['graph_width'] * 0.55) + 0.01);
@@ -567,18 +573,22 @@ abstract class pts_graph_core
 
 			if($this->i['graph_orientation'] == 'HORIZONTAL')
 			{
-				if($this->is_multi_way_comparison && count($this->results) > 1)
+				if($this->i['is_multi_way_comparison'] && count($this->results) > 1)
 				{
 					$longest_string = explode(' - ', $longest_identifier);
 					$longest_string = pts_strings::find_longest_string($longest_string);
-					$rotated_text = round(self::text_string_width($longest_string, $this->i['identifier_size']) * 0.96);
-					$per_identifier_height = 26; // default
+					$per_identifier_height = 22; // default
+					if($this->test_result->test_result_buffer->get_count() > 9)
+					{
+						$per_identifier_height = 20; // default
+						$this->i['identifier_size'] = floor($this->i['identifier_size'] * 0.88);
+					}
+					$rotated_text = round(self::text_string_width($longest_string, $this->i['identifier_size']) * 0.9);
 
-					if(ceil($rotated_text * 1.25) >= floor($per_identifier_height * count($this->results)))
+					if(ceil($rotated_text * 1.2) >= floor($per_identifier_height * count($this->results)))
 					{
 						// this is to avoid having a rotated text bar overrun other results
-						// XXX the 50 number might be too hacky
-						$per_identifier_height = max(30, ceil($rotated_text / count($this->results) * 1.1));
+						$per_identifier_height = max(($per_identifier_height + 2), ceil($rotated_text / count($this->results)));
 					}
 				}
 				else if(count($this->results) > 3)
@@ -604,7 +614,7 @@ abstract class pts_graph_core
 					}
 				}
 
-				$num_identifiers = $this->test_result->test_result_buffer->get_count() + ($this->is_multi_way_comparison ? 2 : 0);
+				$num_identifiers = $this->test_result->test_result_buffer->get_count() + ($this->i['is_multi_way_comparison'] ? 2 : 0);
 				$this->i['graph_top_end'] = $this->i['top_start'] + ($num_identifiers * $per_identifier_height);
 				// $this->i['top_end_bottom']
 				$this->i['graph_height'] = $this->i['graph_top_end'] + 25 + $bottom_heading;
@@ -665,28 +675,21 @@ abstract class pts_graph_core
 	}
 	protected function render_graph_heading($with_version = true)
 	{
-		$href = null;
-
-		if($this->test_identifier != null)
-		{
-			$href = 'https://openbenchmarking.org/test/' . $this->test_identifier;
-		}
-
 		// Default to NORMAL
 		if($this->i['iveland_view'])
 		{
 			$this->svg_dom->add_element('rect', array('x' => 0, 'y' => 0, 'width' => $this->i['graph_width'], 'height' => $this->i['top_heading_height'], 'fill' => self::$c['color']['main_headers']));
 
-			if(isset($this->graph_title[36]))
+			if(isset($this->i['graph_title'][36]))
 			{
 				// If it's a long string make sure it won't run over the side...
-				while(self::text_string_width($this->graph_title, self::$c['size']['headers']) > ($this->i['graph_left_end'] - 20))
+				while(self::text_string_width($this->i['graph_title'], self::$c['size']['headers']) > ($this->i['graph_left_end'] - 20))
 				{
 					self::$c['size']['headers'] -= 0.5;
 				}
 			}
 
-			$this->svg_dom->add_text_element($this->graph_title, array('x' => 6, 'y' => (self::$c['size']['headers'] + 2), 'font-size' => self::$c['size']['headers'], 'fill' => self::$c['color']['background'], 'text-anchor' => 'start', 'xlink:show' => 'new', 'xlink:href' => $href, 'font-weight' => 'bold'));
+			$this->svg_dom->add_text_element($this->i['graph_title'], array('x' => 6, 'y' => (self::$c['size']['headers'] + 2), 'font-size' => self::$c['size']['headers'], 'fill' => self::$c['color']['background'], 'text-anchor' => 'start', 'xlink:show' => 'new', 'xlink:href' => $this->i['header_link'], 'font-weight' => 'bold'));
 
 			foreach($this->graph_sub_titles as $i => $sub_title)
 			{
@@ -705,7 +708,7 @@ abstract class pts_graph_core
 		}
 		else
 		{
-			$this->svg_dom->add_text_element($this->graph_title, array('x' => round($this->i['graph_left_end'] / 2), 'y' => (self::$c['size']['headers'] + 2), 'font-size' => self::$c['size']['headers'], 'fill' => self::$c['color']['main_headers'], 'text-anchor' => 'middle', 'xlink:show' => 'new', 'xlink:href' => $href));
+			$this->svg_dom->add_text_element($this->i['graph_title'], array('x' => round($this->i['graph_left_end'] / 2), 'y' => (self::$c['size']['headers'] + 2), 'font-size' => self::$c['size']['headers'], 'fill' => self::$c['color']['main_headers'], 'text-anchor' => 'middle', 'xlink:show' => 'new', 'xlink:href' => $this->i['header_link']));
 
 			foreach($this->graph_sub_titles as $i => $sub_title)
 			{
@@ -770,9 +773,9 @@ abstract class pts_graph_core
 			}
 		}
 
-		if(!empty($this->graph_y_title) && $this->i['hide_y_title'] == false)
+		if(!empty($this->i['graph_y_title']) && $this->i['hide_y_title'] == false)
 		{
-			$str = $this->graph_y_title;
+			$str = $this->i['graph_y_title'];
 			$offset = 0;
 
 			if($this->i['graph_proportion'] != null)
@@ -830,7 +833,13 @@ abstract class pts_graph_core
 			$tick_width = round(($left_end - $left_start) / $this->i['mark_count']);
 			$display_value = $this->i['graph_min_value'];
 
-			$g = $this->svg_dom->make_g(array('font-size' => self::$c['size']['tick_mark'], 'fill' => self::$c['color']['text'], 'text-anchor' => 'middle'));
+			$tick_font_size = self::$c['size']['tick_mark'];
+			while($this->i['graph_max_value'] > 100000 && self::text_string_width($this->i['graph_max_value'], $tick_font_size) + 4 > $tick_width)
+			{
+				$tick_font_size--;
+			}
+
+			$g = $this->svg_dom->make_g(array('font-size' => $tick_font_size, 'fill' => self::$c['color']['text'], 'text-anchor' => 'middle'));
 			$g_lines = $this->svg_dom->make_g(array('stroke' => self::$c['color']['body'], 'stroke-width' => 1));
 			for($i = 0; $i < $this->i['mark_count']; $i++)
 			{
@@ -892,7 +901,7 @@ abstract class pts_graph_core
 	}
 	protected function graph_key_height()
 	{
-		if((count($this->results) < 2 || $this->i['show_graph_key'] == false) && !$this->is_multi_way_comparison) // TODO likely should be OR
+		if((count($this->results) < 2 || $this->i['show_graph_key'] == false) && !$this->i['is_multi_way_comparison']) // TODO likely should be OR
 		{
 			return 0;
 		}
@@ -975,9 +984,9 @@ abstract class pts_graph_core
 	protected function adjust_color($identifier, $paint_color)
 	{
 		// Adjust the color based on the identifier and the content of value_highlights
-		if(array_key_exists($identifier, $this->value_highlights) || in_array($identifier, $this->value_highlights))
+		if($this->i['highlight_values'] && (array_key_exists($identifier, $this->i['highlight_values']) || in_array($identifier, $this->i['highlight_values'])))
 		{
-			$color = $this->value_highlights[$identifier];
+			$color = isset($this->i['highlight_values'][$identifier]) ? $this->i['highlight_values'][$identifier] : null;
 			$paint_color = empty($color) ? $this->darken_color($paint_color) : $color;
 		}
 		return $paint_color;
