@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2017, Phoronix Media
-	Copyright (C) 2008 - 2017, Michael Larabel
+	Copyright (C) 2008 - 2018, Phoronix Media
+	Copyright (C) 2008 - 2018, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -55,6 +55,18 @@ class pts_network
 
 		return $contents;
 	}
+	public static function can_reach_phoronix_test_suite_com()
+	{
+		return pts_network::http_get_contents('http://www.phoronix-test-suite.com/PTS') == 'PTS';
+	}
+	public static function can_reach_openbenchmarking_org()
+	{
+		return pts_network::http_get_contents('http://openbenchmarking.org/PTS') == 'PTS';
+	}
+	public static function can_reach_phoronix_net()
+	{
+		return pts_network::http_get_contents('http://phoronix.net/PTS') == 'PTS';
+	}
 	public static function http_upload_via_post($url, $to_post_data, $supports_proxy = true)
 	{
 		if(!pts_network::network_support_available())
@@ -62,8 +74,7 @@ class pts_network
 			return false;
 		}
 
-		$upload_data = http_build_query($to_post_data);
-		$http_parameters = array('http' => array('method' => 'POST', 'content' => $upload_data));
+		$http_parameters = array('http' => array('method' => 'POST', 'content' => http_build_query($to_post_data)));
 		if($supports_proxy)
 		{
 			$stream_context = pts_network::stream_context_create($http_parameters);
@@ -83,7 +94,6 @@ class pts_network
 		{
 			return false;
 		}
-
 		if(strpos($download, '://') === false)
 		{
 			$download = 'http://' . $download;
@@ -94,8 +104,11 @@ class pts_network
 			$download = str_replace('https://', 'http://', $download);
 		}
 
-		if(function_exists('curl_init') && stripos(PTS_PHP_VERSION, 'hiphop') === false)
+		if(PTS_IS_CLIENT && strpos(phodevi::read_property('system', 'operating-system'), ' 7') === false && function_exists('curl_init') && stripos(PTS_PHP_VERSION, 'hiphop') === false)
 		{
+			// XXX: RHEL/EL 7.6 PHP packages introduced a segv when using CURL... Until that's resolved, just blacklist " 7"
+			// as unknown when it will be fixed, but at least there is non-CURL codepath supported fine
+			// " 7" is a bit liberal but also hard due to various EL7 downstreams
 			// XXX: Facebook HipHop HHVM currently seems to have problems with PHP CURL
 			$return_state = pts_network::curl_download($download, $to);
 		}
@@ -214,6 +227,9 @@ class pts_network
 			$parameters = array();
 		}
 
+		$parameters['ssl']['verify_peer'] = false;
+		$parameters['ssl']['verify_peer_name'] = false;
+
 		if($proxy_address == false && $proxy_port == false && self::$network_proxy)
 		{
 			$proxy_address = self::$network_proxy['address'];
@@ -398,8 +414,8 @@ class pts_network
 		// try and get the device with the default route
 		if ($ip = pts_client::executable_in_path('ip')) {
 			$out = shell_exec("$ip route 2>&1");
-			$start = strpos($out,' dev ')+5;
-			$dev = substr($out, $start, strpos($out,' ', $start) - $start);
+			$start = strpos($out, ' dev ') + 5;
+			$dev = substr($out, $start, strpos($out, ' ', $start) - $start);
 		}
 
 		// we grab the last field of the `netstat -nr` output, betting on *bsd not expiring it's default route
@@ -453,44 +469,92 @@ class pts_network
 				}
 			}
 		}
+		else if(phodevi::is_windows())
+		{
+			$ipconfig = shell_exec('ipconfig');
+			$offset = 0;
+
+			while(($ipv4_pos = strpos($ipconfig, 'IPv4 Address.', $offset)) !== false)
+			{
+				$ipv4 = substr($ipconfig, $ipv4_pos);
+				$ipv4 = substr($ipv4, strpos($ipv4, ': ') + 2);
+				$ipv4 = substr($ipv4, 0, strpos($ipv4, "\n"));
+				$local_ip = trim($ipv4);
+
+				if($local_ip != '127.0.0.1' && $local_ip != null && strpos($local_ip, '169.254') === false)
+				{
+					break;
+				}
+				$offset = $ipv4_pos + 3;
+			}
+		}
 		else if(pts_client::executable_in_path('hostname'))
 		{
-			$hostname_i = trim(shell_exec('hostname -I 2>&1'));
-			if(strpos($hostname_i, ' ') === false)
+			$hostname_i = explode(' ', trim(shell_exec('hostname -I 2>&1')));
+			$hostname_i = array_shift($hostname_i);
+			if(count(explode('.', $hostname_i)) == 4)
 			{
 				$local_ip = $hostname_i;
 			}
 		}
 
+		if(empty($local_ip) && function_exists('net_get_interfaces'))
+		{
+			// The below code should work as of net_get_interfaces() as of PHP 7.3 in cross-platform manner
+			$net_interfaces = net_get_interfaces();
+			foreach($net_interfaces as $interface => $interface_info)
+			{
+				if(isset($interface_info['unicast'][1]['address']) && !empty($interface_info['unicast'][1]['address']) && $interface_info['unicast'][1]['address'] != '127.0.0.1')
+				{
+					$local_ip = $interface_info['unicast'][1]['address'];
+					break;
+				}
+			}
+		}
+
 		return $local_ip;
 	}
 	public static function get_network_mac()
 	{
 		$mac = false;
 
-		if ($interface = self::get_active_network_interface())
+		if(phodevi::is_linux())
 		{
-			$addr =  "/sys/class/net/$interface/address";
-			if(is_file($addr))
+			if($interface = self::get_active_network_interface())
 			{
-				$mac = pts_file_io::file_get_contents($addr);
-			}
-		}
-
-		if (empty($mac)) {
-			foreach(pts_file_io::glob('/sys/class/net/*/operstate') as $net_device_state)
-			{
-				if(pts_file_io::file_get_contents($net_device_state) == 'up')
+				$addr =  "/sys/class/net/$interface/address";
+				if(is_file($addr))
 				{
-					$addr = dirname($net_device_state) . '/address';
-					if(is_file($addr))
+					$mac = pts_file_io::file_get_contents($addr);
+				}
+			}
+
+			if(empty($mac))
+			{
+				foreach(pts_file_io::glob('/sys/class/net/*/operstate') as $net_device_state)
+				{
+					if(pts_file_io::file_get_contents($net_device_state) == 'up')
 					{
-						$mac = pts_file_io::file_get_contents($addr);
-						break;
+						$addr = dirname($net_device_state) . '/address';
+						if(is_file($addr))
+						{
+							$mac = pts_file_io::file_get_contents($addr);
+							break;
+						}
 					}
 				}
 			}
 		}
+		else if(phodevi::is_windows())
+		{
+			$getmac = shell_exec('getmac');
+			$getmac = trim(substr($getmac, strpos($getmac, "\n", strpos($getmac, '======='))));
+			$getmac = substr($getmac, 0, strpos($getmac, ' '));
+			if(strlen($getmac) <= 17)
+			{
+				$mac = str_replace('-', ':', $getmac);
+			}
+		}
 
 		if(empty($mac) && ($ifconfig = pts_client::executable_in_path('ifconfig')))
 		{
@@ -648,13 +712,13 @@ class pts_network
 	}
 	public static function find_zeroconf_phoromatic_servers($find_multiple = false)
 	{
+		$hosts = $find_multiple ? array() : null;
+
 		if(!pts_network::network_support_available())
 		{
-			return null;
+			return $hosts;
 		}
 
-		$hosts = $find_multiple ? array() : null;
-
 		if(PTS_IS_CLIENT && pts_client::executable_in_path('avahi-browse'))
 		{
 			$avahi_browse = explode(PHP_EOL, shell_exec('avahi-browse -p -r -t _http._tcp 2>&1'));
