@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2010 - 2017, Phoronix Media
-	Copyright (C) 2010 - 2017, Michael Larabel
+	Copyright (C) 2010 - 2019, Phoronix Media
+	Copyright (C) 2010 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -58,6 +58,10 @@ class pts_openbenchmarking
 			'screen_resolution' => array('gpu', 'screen-resolution-string')
 			);
 	}
+	public static function openbenchmarking_standards_path()
+	{
+		return PTS_CORE_PATH . 'openbenchmarking.org/';
+	}
 	public static function is_valid_gsid_format($gsid)
 	{
 		$gsid_valid = false;
@@ -114,6 +118,40 @@ class pts_openbenchmarking
 
 		return $is_id;
 	}
+	public static function result_uploads_from_this_ip()
+	{
+		$results = array();
+
+		if(pts_network::internet_support_available())
+		{
+			$json_response = pts_openbenchmarking::make_openbenchmarking_request('result_uploads_from_this_ip');
+			$json_response = json_decode($json_response, true);
+
+			if(is_array($json_response) && isset($json_response['uploads']['public_ids']))
+			{
+				$results = $json_response['uploads']['public_ids'];
+			}
+		}
+
+		return $results;
+	}
+	public static function possible_phoromatic_servers()
+	{
+		$results = array();
+
+		if(pts_network::internet_support_available())
+		{
+			$json_response = pts_openbenchmarking::make_openbenchmarking_request('phoromatic_server_relay_check');
+			$json_response = json_decode($json_response, true);
+
+			if(is_array($json_response) && isset($json_response['phoromatic']['possible_servers']))
+			{
+				$results = $json_response['phoromatic']['possible_servers'];
+			}
+		}
+
+		return $results;
+	}
 	public static function clone_openbenchmarking_result(&$id, $return_xml = false)
 	{
 		if(!pts_network::internet_support_available())
@@ -224,19 +262,23 @@ class pts_openbenchmarking
 			// OpenSSL seems to have problems on OpenIndiana at least, TODO: investigate
 			// Using a proxy seems to have problems for HTTPS
 			// TODO XXX
-			//$host = ((extension_loaded('openssl') && getenv('NO_OPENSSL') == false && php_uname('s') == 'Linux' && (!PTS_IS_CLIENT || !pts_network::is_proxy_setup())) ? 'https://' : 'http://') . 'openbenchmarking.org/';
-			$host = 'http://openbenchmarking.org/';
+			$host = ((extension_loaded('openssl') && getenv('NO_OPENSSL') == false && (!PTS_IS_CLIENT || !pts_network::is_proxy_setup())) ? 'https://' : 'http://') . 'openbenchmarking.org/';
+			//$host = 'http://openbenchmarking.org/';
 		}
 
 		return $host;
 	}
 	public static function report_repository_index_updates($repo, $old_index, $new_index)
 	{
-		if(isset($new_index['tests']) && isset($old_index['tests']) && $new_index['tests'] != $old_index['tests'])
+		if(isset($new_index['tests']) && isset($old_index['tests']) && ($new_index['tests'] != $old_index['tests'] || $new_index['suites'] != $old_index['suites']))
 		{
-			$new_test_count = count($new_index['tests']);
+			$test_versions_count = 0;
+			foreach($new_index['tests'] as $t)
+			{
+				$test_versions_count += count($t['versions']);
+			}
 			echo pts_client::cli_colored_text('Updated OpenBenchmarking.org Repository Index', 'green', true) . PHP_EOL;
-			echo pts_client::cli_colored_text($repo . ': ' . count($new_index['tests']) . ' Distinct Tests, ' . count($new_index['suites']) . ' Suites', 'green', true) . PHP_EOL;
+			echo pts_client::cli_colored_text($repo . ': ' . pts_strings::plural_handler(count($new_index['tests']), 'Distinct Test') . ', ' . pts_strings::plural_handler($test_versions_count, 'Test Version') . ', ' . pts_strings::plural_handler(count($new_index['suites']), 'Suite'), 'green', true) . PHP_EOL;
 			$table = array();
 			foreach(array_keys($new_index['tests']) as $test)
 			{
@@ -253,9 +295,42 @@ class pts_openbenchmarking
 					}
 				}
 			}
+
+
+			if(isset($new_index['suites']) && isset($old_index['suites']) && $new_index['suites'] != $old_index['suites'])
+			{
+				foreach(array_keys($new_index['suites']) as $suite)
+				{
+					if(!isset($old_index['suites'][$suite]))
+					{
+						$table[] = array(pts_client::cli_just_bold('New Suite Available: '), $repo . '/' . $suite, pts_client::cli_colored_text('v' . array_shift($new_index['suites'][$suite]['versions']), 'gray'));
+					}
+					else if($new_index['suites'][$suite]['versions'] != $old_index['suites'][$suite]['versions'])
+					{
+						$version_diff = array_diff($new_index['suites'][$suite]['versions'], $old_index['suites'][$suite]['versions']);
+						if(!empty($version_diff))
+						{
+							$table[] = array(pts_client::cli_just_bold('Updated Suite Available: '), $repo . '/' . $suite, pts_client::cli_colored_text('v' . array_shift($version_diff), 'gray'));
+						}
+					}
+				}
+			}
 			echo pts_user_io::display_text_table($table) . PHP_EOL;
 		}
 	}
+	public static function get_generated_time_from_index($index_file)
+	{
+		$generated_time = -1;
+		if(is_file($index_file))
+		{
+			$repo_index = json_decode(file_get_contents($index_file), true);
+			if(isset($repo_index['main']['generated']))
+			{
+				$generated_time = $repo_index['main']['generated'];
+			}
+		}
+		return $generated_time;
+	}
 	public static function refresh_repository_lists($repos = null, $force_refresh = false)
 	{
 		if($repos == null)
@@ -298,26 +373,29 @@ class pts_openbenchmarking
 				$repo_index = json_decode(file_get_contents($index_file), true);
 				$generated_time = $repo_index['main']['generated'];
 
-				// Refreshing the indexes once every few days should be suffice
-				// Refresh approximately every three days by default
-				$index_cache_ttl = 3;
-				if(PTS_IS_CLIENT && ($config_ttl = pts_config::read_user_config('PhoronixTestSuite/Options/OpenBenchmarking/IndexCacheTTL')))
+				if($force_refresh == false)
 				{
-					if($config_ttl === 0)
+					// Refreshing the indexes once every few days should be suffice
+					// Refresh approximately every three days by default
+					$index_cache_ttl = 3;
+					if(PTS_IS_CLIENT && ($config_ttl = pts_config::read_user_config('PhoronixTestSuite/Options/OpenBenchmarking/IndexCacheTTL')))
 					{
-						// if the value is 0, only rely upon manual refreshes
+						if($config_ttl === 0)
+						{
+							// if the value is 0, only rely upon manual refreshes
+							continue;
+						}
+						else if(is_numeric($config_ttl) && $config_ttl >= 1)
+						{
+							$index_cache_ttl = $config_ttl;
+						}
+					}
+
+					if($generated_time > (time() - (86400 * $index_cache_ttl)) && (!defined('FIRST_RUN_ON_PTS_UPGRADE') || FIRST_RUN_ON_PTS_UPGRADE == false))
+					{
+						// The index is new enough
 						continue;
 					}
-					else if(is_numeric($config_ttl) && $config_ttl >= 1)
-					{
-						$index_cache_ttl = $config_ttl;
-					}
-				}
-
-				if($generated_time > (time() - (86400 * $index_cache_ttl)) && $force_refresh == false && (!defined('FIRST_RUN_ON_PTS_UPGRADE') || FIRST_RUN_ON_PTS_UPGRADE == false))
-				{
-					// The index is new enough
-					continue;
 				}
 
 				$old_index = $repo_index;
@@ -373,7 +451,7 @@ class pts_openbenchmarking
 
 				if(!isset($reported_read_failure_notice[$repo_name]) && PTS_IS_CLIENT)
 				{
-					trigger_error('Failed To Fetch OpenBenchmarking.org Repository Data: ' . $repo_name, E_USER_WARNING);
+					trigger_error('Failed To Fetch OpenBenchmarking.org Repository Data: ' . $repo_name . '. If this issue persists, contact support@phoronix-test-suite.com.', E_USER_WARNING);
 					$reported_read_failure_notice[$repo_name] = true;
 				}
 			}
@@ -385,7 +463,13 @@ class pts_openbenchmarking
 	}
 	public static function linked_repositories()
 	{
-		$repos = array('local', 'pts', 'system');
+		$repos = array('local');
+
+		if(PTS_IS_CLIENT && phodevi::is_windows())
+		{
+			// Various windows tests for compatibility where there isn't mainline support in the test profile otherwise
+			array_unshift($repos, 'windows');
+		}
 
 		if(PTS_IS_CLIENT && pts_openbenchmarking_client::user_name() != false)
 		{
@@ -454,13 +538,17 @@ class pts_openbenchmarking
 
 		return $index_file;
 	}
-	public static function download_test_profile($qualified_identifier, $download_location = null)
+	public static function is_test_profile_downloaded($qualified_identifier)
+	{
+		return is_file(PTS_TEST_PROFILE_PATH . $qualified_identifier . '/test-definition.xml');
+	}
+	public static function download_test_profile($qualified_identifier, $download_location = null, $cache_check = false)
 	{
 		if(empty($download_location))
 		{
 			$download_location = PTS_TEST_PROFILE_PATH;
 		}
-		if(is_file($download_location . $qualified_identifier . '/test-definition.xml'))
+		if(is_file($download_location . $qualified_identifier . '/test-definition.xml') && !$cache_check)
 		{
 			return true;
 		}
@@ -506,12 +594,18 @@ class pts_openbenchmarking
 					$test_profile = $b64;
 				}
 
-				file_put_contents($file, $test_profile);
+				if(!empty($test_profile))
+				{
+					file_put_contents($file, $test_profile);
+				}
 			}
 
 			if(PTS_IS_CLIENT && !is_file($file))
 			{
-				trigger_error('Network support is needed to obtain ' . $qualified_identifier . ' data.' . PHP_EOL, E_USER_ERROR);
+				if(!defined('PHOROMATIC_SERVER'))
+				{
+					trigger_error('Unable to obtain ' . $qualified_identifier . ' from OpenBenchmarking.org. If this issue persists, contact support@phoronix-test-suite.com.' . PHP_EOL, E_USER_ERROR);
+				}
 				return false;
 			}
 		}
@@ -537,28 +631,49 @@ class pts_openbenchmarking
 
 		return false;
 	}
-	public static function phoromatic_server_ob_cache_request($type_request, $repo = null, $test = null)
+	public static function remote_test_profile_check($identifier)
 	{
+		// See if the test (e.g. a local/ test) is available for download from Phoromatic Servers that is not part of an Openbenchmarking.org repo index
 		if(PTS_IS_CLIENT == false)
 		{
-			return null;
+			return false;
 		}
-
-		$archived_servers = pts_client::available_phoromatic_servers();
-
-		foreach($archived_servers as $archived_server)
+		$is_test = self::phoromatic_server_ob_cache_request('is_test', null, $identifier);
+		if(!empty($is_test) && strpos($is_test, ' ') === false && strpos($is_test, '..') === false && strpos($is_test, '/') !== false)
 		{
-			$cache = pts_network::http_get_contents('http://' . $archived_server['ip'] . ':' . $archived_server['http_port'] . '/openbenchmarking-cache.php?' . $type_request . '&repo=' . $repo . '&test=' . $test);
-
-			if(!empty($cache))
+			if($test_profile = self::phoromatic_server_ob_cache_request('test', substr($is_test, 0, strpos($is_test, '/')), substr($is_test, strpos($is_test, '/') + 1)))
 			{
-				return $cache;
+				$test_zip = base64_decode($test_profile);
+				if(!empty($test_zip))
+				{
+					$zip_file = tempnam(sys_get_temp_dir(), 'phoromatic-zip');
+					file_put_contents($zip_file, $test_zip);
+
+					// Extract the temp zip
+					$download_location = PTS_TEST_PROFILE_PATH;
+					if(!is_file($download_location . $is_test . '/test-definition.xml') && is_file($zip_file))
+					{
+						// extract it
+						pts_file_io::mkdir($download_location . dirname($is_test));
+						pts_file_io::mkdir($download_location . $is_test);
+						pts_compression::zip_archive_extract($zip_file, $download_location . $is_test);
+
+						if(is_file(PTS_TEST_PROFILE_PATH . $is_test . '/test-definition.xml'))
+						{
+							return true;
+						}
+					}
+					unlink($zip_file);
+				}
 			}
 		}
-
+		return false;
+	}
+	public static function phoromatic_server_ob_cache_request($type_request, $repo = null, $test = null)
+	{
 		return null;
 	}
-	public static function available_tests($download_tests = true, $all_versions = false, $append_versions = false, $show_deprecated_tests = false)
+	public static function available_tests($download_tests = true, $all_versions = false, $append_versions = false, $show_deprecated_tests = false, $only_show_available_cached_tests = false)
 	{
 		$available_tests = array();
 
@@ -596,6 +711,21 @@ class pts_openbenchmarking
 							}
 						}
 
+						if($only_show_available_cached_tests)
+						{
+							if(!pts_openbenchmarking::is_test_profile_downloaded($identifier . '-' . $version))
+							{
+								// Without Internet, won't be able to download test, so don't show it
+								continue;
+							}
+							$test_profile = new pts_test_profile($identifier . '-' . $version);
+							if(pts_test_install_request::test_files_available_via_cache($test_profile) == false)
+							{
+								// Without Internet, only show tests where files are local or in an available cache
+								continue;
+							}
+						}
+
 						$available_tests[] = $repo . '/' . $identifier . ($append_versions ? '-' . $version : null);
 					}
 				}
@@ -605,7 +735,7 @@ class pts_openbenchmarking
 
 		return $available_tests;
 	}
-	public static function available_suites($download_suites = true)
+	public static function available_suites($download_suites = true, $only_show_maintained_suites = false)
 	{
 		$available_suites = array();
 
@@ -617,6 +747,11 @@ class pts_openbenchmarking
 			{
 				foreach(array_keys($repo_index['suites']) as $identifier)
 				{
+					if($only_show_maintained_suites && strtotime($repo_index['suites'][$identifier]['last_updated']) < (time() - (86400 * 365 * 4)))
+					{
+						continue;
+					}
+
 					if($download_suites && pts_network::internet_support_available())
 					{
 						$version = array_shift($repo_index['suites'][$identifier]['versions']);
@@ -632,13 +767,13 @@ class pts_openbenchmarking
 
 		return $available_suites;
 	}
-	public static function download_test_suite($qualified_identifier, $download_location = null)
+	public static function download_test_suite($qualified_identifier, $download_location = null, $cache_check = false)
 	{
 		if(empty($download_location))
 		{
 			$download_location = PTS_TEST_SUITE_PATH;
 		}
-		if(is_file($download_location . $qualified_identifier . '/suite-definition.xml'))
+		if(is_file($download_location . $qualified_identifier . '/suite-definition.xml') && !$cache_check)
 		{
 			return true;
 		}
@@ -651,6 +786,10 @@ class pts_openbenchmarking
 			$hash_json = json_decode($hash_json, true);
 			$hash_check = isset($hash_json['openbenchmarking']['suite']['hash']) ? $hash_json['openbenchmarking']['suite']['hash'] : null;  // should also check for ['openbenchmarking']['suite']['error'] problems
 		}
+		else
+		{
+			$hash_check = null;
+		}
 
 		if(!is_file($file))
 		{
@@ -677,12 +816,18 @@ class pts_openbenchmarking
 					$test_suite = $b64;
 				}
 
-				file_put_contents($file, $test_suite);
+				if(!empty($test_suite))
+				{
+					file_put_contents($file, $test_suite);
+				}
 			}
 
 			if(PTS_IS_CLIENT && !is_file($file))
 			{
-				trigger_error('Network support is needed to obtain ' . $qualified_identifier . ' data.' . PHP_EOL, E_USER_ERROR);
+				if(!defined('PHOROMATIC_SERVER'))
+				{
+					trigger_error('Unable to obtain ' . $qualified_identifier . ' from OpenBenchmarking.org.  If this issue persists, contact support@phoronix-test-suite.com.' . PHP_EOL, E_USER_ERROR);
+				}
 				return false;
 			}
 		}
@@ -890,7 +1035,11 @@ class pts_openbenchmarking
 	}
 	public static function upload_test_result(&$object, $return_json_data = false, $prompts = true)
 	{
-		return pts_openbenchmarking_client::upload_test_result($object, $return_json_data, $prompts);
+		return pts_openbenchmarking_upload::upload_test_result($object, $return_json_data, $prompts);
+	}
+	public static function ob_upload_support_available()
+	{
+		return pts_auto_load_class('pts_openbenchmarking_upload') && pts_config::read_bool_config('PhoronixTestSuite/Options/OpenBenchmarking/AllowResultUploadsToOpenBenchmarking', 'TRUE');
 	}
 }
 
