@@ -39,6 +39,10 @@ class pts_test_result_buffer
 	}
 	public function __clone()
 	{
+		foreach($this->buffer_items as $i => $v)
+		{
+			$this->buffer_items[$i] = clone $this->buffer_items[$i];
+		}
 	}
 	public function get_buffer_items()
 	{
@@ -113,7 +117,14 @@ class pts_test_result_buffer
 	}
 	public function rename($from, $to)
 	{
-		if($from == null && count($this->buffer_items) == 1)
+		if($from == 'PREFIX')
+		{
+			foreach($this->buffer_items as &$buffer_item)
+			{
+				$buffer_item->reset_result_identifier($to . ': ' . $buffer_item->get_result_identifier());
+			}
+		}
+		else if($from == null && count($this->buffer_items) == 1)
 		{
 			foreach($this->buffer_items as &$buffer_item)
 			{
@@ -295,13 +306,33 @@ class pts_test_result_buffer
 
 		return $identifier;
 	}
+	public function result_identifier_differences_only_numeric()
+	{
+		if(!isset($this->buffer_items[0]))
+		{
+			return false;
+		}
+
+		$first_result = pts_strings::remove_from_string($this->buffer_items[0]->get_result_identifier(), pts_strings::CHAR_NUMERIC | pts_strings::CHAR_DECIMAL | pts_strings::CHAR_DASH);
+		for($i = 1;  $i < count($this->buffer_items); $i++)
+		{
+			$result = pts_strings::remove_from_string($this->buffer_items[$i]->get_result_identifier(), pts_strings::CHAR_NUMERIC | pts_strings::CHAR_DECIMAL | pts_strings::CHAR_DASH);
+			if($result != $first_result)
+			{
+				return false;
+			}
+		}
+		return true;
+	}
 	public function get_max_value($return_identifier = false)
 	{
 		$value = 0;
 		$max_id = null;
+		$precision = 2;
 
 		foreach($this->buffer_items as &$buffer_item)
 		{
+			$precision = max($precision, pts_math::get_precision($buffer_item->get_result_value()));
 			if($buffer_item->get_result_value() > $value)
 			{
 				$value = $buffer_item->get_result_value();
@@ -309,7 +340,7 @@ class pts_test_result_buffer
 			}
 		}
 
-		return $return_identifier ? $max_id : $value;
+		return $return_identifier ? $max_id : pts_math::set_precision($value, $precision);
 	}
 	public function get_min_value($return_identifier = false)
 	{
@@ -327,6 +358,18 @@ class pts_test_result_buffer
 
 		return $return_identifier ? $max_id : $value;
 	}
+	public function has_run_with_multiple_samples()
+	{
+		foreach($this->buffer_items as &$buffer_item)
+		{
+			if($buffer_item->get_sample_count() > 1)
+			{
+				return true;
+			}
+		}
+
+		return false;
+	}
 	public function get_value_from_identifier($result_identifier)
 	{
 		foreach($this->buffer_items as &$buffer_item)
@@ -371,6 +414,46 @@ class pts_test_result_buffer
 			}
 		}
 	}
+	public function adjust_precision($precision = 'auto')
+	{
+		if($precision == 'auto')
+		{
+			// For very large results, little point in keeping the precision...
+			$min_value = $this->get_min_value();
+			$precision = -1;
+			if($min_value >= 100)
+			{
+				$precision = 0;
+			}
+			if($min_value >= 10)
+			{
+				$precision = 2;
+			}
+
+			$current_precision = 0;
+			foreach($this->buffer_items as &$buffer_item)
+			{
+				if(is_numeric(($val = $buffer_item->get_result_value())))
+				{
+					$current_precision = max($current_precision, pts_math::get_precision($val));
+				}
+			}
+
+			$precision = $precision == -1 ? $current_precision : min($precision, $current_precision);
+		}
+
+		if(is_numeric($precision))
+		{
+			foreach($this->buffer_items as &$buffer_item)
+			{
+				if(is_numeric(($val = $buffer_item->get_result_value())))
+				{
+					$buffer_item->reset_result_value(pts_math::set_precision($val, $precision), false);
+				}
+			}
+
+		}
+	}
 	public function get_values()
 	{
 		$values = array();
