@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2009 - 2017, Phoronix Media
-	Copyright (C) 2009 - 2017, Michael Larabel
+	Copyright (C) 2009 - 2019, Phoronix Media
+	Copyright (C) 2009 - 2019, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -42,8 +42,14 @@ class pts_test_result_buffer_item
 	{
 		$this->result_identifier = $identifier;
 	}
-	public function reset_result_value($value)
+	public function reset_result_value($value, $set_precision = true)
 	{
+		if(is_numeric($this->result_final) && $set_precision && is_numeric($value))
+		{
+			$precision = pts_math::get_precision($this->result_final);
+			$value = round($value, ($precision < 2 ? 2 : $precision));
+		}
+
 		$this->result_final = $value;
 	}
 	public function reset_raw_value($value)
@@ -70,6 +76,14 @@ class pts_test_result_buffer_item
 	{
 		return $this->result_raw;
 	}
+	public function get_result_raw_array()
+	{
+		return explode(':', $this->result_raw);
+	}
+	public function get_sample_count()
+	{
+		return count($this->get_result_raw_array());
+	}
 	public function get_result_json()
 	{
 		if($this->result_json != null && !is_array($this->result_json))
@@ -87,6 +101,13 @@ class pts_test_result_buffer_item
 	{
 		$a = $a->get_result_value();
 		$b = $b->get_result_value();
+		if(strpos($a, ',') != false && strpos($b, ',') != false)
+		{
+			$a = explode(',', $a);
+			$b = explode(',', $b);
+			$a = pts_math::arithmetic_mean($a);
+			$b = pts_math::arithmetic_mean($b);
+		}
 
 		if($a == $b)
 		{
